## Example code to conduct the estimation
require(INLA)
INLA:::inla.dynload.workaround() 
source("zipbk_inla_15b.R")
source("rzi_3.R")
source("ma_el_5c.R")


# load WDS data and divide them by survey year
wds <- read.csv(file="wds_sim_090716.csv",head=T)
wds$STRATUM <- factor(wds$STRATUM) ## define stratum variable
wds$t_POINT_Y <- scale(wds$POINT_Y) ## scale trend for numerical stability
wds$t_DEPTH_B <- scale(wds$DEPTH_B) ## scale bottom detph for numerical stability
l <- split(wds,wds$YEAR_)

# load grid definition from Chesapeake Bay Program
grid <- read.csv(file="cbp_grid_090716.csv",head=T)
grid$STRATUM <- factor(grid$STRATUM) ## define stratum variable
grid$t_POINT_Y <- scale(grid$POINT_Y) ## scale trend for numerical stability
grid$t_DEPTH_B <- scale(grid$DEPTH_B) ## scale bottom detph for numerical stability
grid$Dummy <- 1 ## no segment, bay-wide estimation

# load shoreline boundary
load(file="cbp_boundary_090716.rda")

# Start calibration
result <- lapply(l,function(elmt){
  # Fit working model
  r <- zipbk.inla(query=grid,obs=elmt,fixed=~t_POINT_Y+t_DEPTH_B+STRATUM,random=~s+e,
                    family="zeroinflatedpoisson0",xcoord='POINT_X',ycoord='POINT_Y',
                    wcoord="AREA",bcoord='Dummy',zcoord="ysim", 
                    max.edge=c(5000,1000000),cutoff=1000,boundary=segm.bnd,mesh=NULL,
                    prior.variance.nominal=1,prior.range.nominal=NULL,
                    theta.prior.mean=c(0,0), theta.prior.prec=c(0.1,0.1),
                    unstruct.hyper= 'list(theta=list(prior="loggamma",param=c(0.01,0.01)))',
                    predict=T,verbose=F,
                    control=list(nsamples=399)
                    )
  r$inla <- NULL
  r$samples <- NULL
  
  ## Empirical likelihood-based calibration.
  
  ## observed cpue
  obs.cpue <- elmt[,"ysim"]
  ## data mapping
  require(FNN)
  sampled.grid <- get.knnx(
    query=as.matrix(elmt[,c("POINT_X","POINT_Y")]),
    data=as.matrix(grid[,c("POINT_X","POINT_Y")]),
    k=1)$nn.index[,1]
  fit.cpue <- apply(r$predSample,1,median)
  ## naive inclusion probability
  pi.lst <- ma.el.prop(y=obs.cpue,idx=sampled.grid,Yhat=fit.cpue)
  
  cat("calibration to point estimate (posterior median) \n")
  ## model calibration based estimate
  r.calib <- ma.el(y=obs.cpue,idx=sampled.grid,Yhat=fit.cpue,pi.lst=pi.lst)
  cat("done\n")
  cat("calibration to the full approximate MCMC samples\n")
  out <- apply(r$predSample,2,function(x){
    r <- ma.el(y=obs.cpue,idx=sampled.grid,Yhat=x,pi.lst=pi.lst)
    r$value
  })
  cat("done\n")
  (fit <- mean(out[1,])) ## calibrated mean
  (sigmasq <- var(out[1,]) + mean(out[2,]^2)) ## calibrated standard error
  ## calibration results
  r$value <- c(mean(obs.cpue),sd(obs.cpue),
               r.calib$value[1:2],
               fit,sd(out[1,]),sqrt(mean(out[2,]^2)),sqrt(sigmasq))
  
  ## return;
  r
})

## results
for(i in 1:length(result)){
  out <- result[[i]]$value
  cat("year =",names(l)[i]," mean=",out[1],"se=",out[2]/sqrt(nrow(l[[i]])),
      " calibrated mean=",out[5],
      " calibrated se=",out[8],"\n")
}