## zero truncated distribution simulation
## update extreme small lambda to 1
rzi.pois <- function(n,lambda)
{
  l.lambda <- cbind(rep(1,n),lambda)[,2]
  u <- rpois(n=n,lambda=l.lambda) ## sample from Poisson
  ii <- which(u==0) ## identify zero components
  
  u[ii[which(l.lambda[ii]<0.001)]] <- 1 ## update extremely small mean with 1
  
  ii <- which(u==0)
  r <- u  ## return
  n1 <- length(ii) ## zero values
  ndraw <- n
  #browser()
  while(n1>0){ ## when there are zeros
    u <- rpois(n=n1,lambda=l.lambda[ii]) ## re-sample corresponding entries
    #cat(ndraw," draws, current",n1," zeros,")
    ndraw <- ndraw+n1
    r[ii[which(u>0)]] <- u[u>0] ## update positive values if any
    #cat(sum(u>0)," updates\n")   
    ii <- which(r==0) ## recalculate number of zeros
    n1 <- length(ii) 
  }
  list(samples=r,ndraw=ndraw,ratio=n/ndraw)
}

## rejection sampling from negative binomial truncated at zero
## accept rate is proportional to 1 minus the success probability 
## when success probability is close to 1, this algorithm is very inefficient
rzi.nbinom <- function(n,size,prob)
{
  tmp <- cbind(rep(1,n),size,prob)
  l.size <- tmp[,2]
  l.prob <- tmp[,3]
  
  ## update extreme small success probability with size / prob
  u <- l.size*(1-l.prob)/l.prob
  u[l.prob>1e-3] <- rnbinom(
    n=sum(l.prob>1e-3),size=l.size[l.prob>1e-3],
    prob=l.prob[l.prob>1e-3])
  #u <- rnbinom(n=n,size=l.size,prob=l.prob)
  r <- u ## return
  
  ## update extreme large success probability with 1
  ii <- which(u==0)
  r[ii[which(l.prob[ii]>0.99)]] <- 1 
  
  ii <- which(r==0)
  n1 <- length(ii) ## zero values
  ndraw <- n
  while(n1>0){ ## when there are zeros
    u <- rnbinom(n=n1,size=l.size[ii],prob=l.prob[ii]) ##re-sample corresponding entries
    #cat(ndraw," draws, current",n1," zeros,")
    ndraw <- ndraw + n1
    r[ii[which(u>0)]] <- u[u>0]
    #cat(sum(u>0)," updates\n")
    ii <- which(r==0) ## recalculate number of zeroes
    n1 <- length(ii)
  }
  list(samples=r,ndraw=ndraw,ratio=n/ndraw)
}

test <- function()
{
  rm(list=ls())
  source('rzi_2.R')
  n <- 2000
  prob <- 9999/10000
  x <- rzi.nbinom(n,size=1,prob=prob)
  (x$ratio)
  dx <- function(x){
    p <- pnbinom(0,size=1,prob=prob)
    dnbinom(x,size=1,prob=prob)/(1-p)
  }
  ux <- sort(unique(x$samples))
  oldpar <- par(pty="s",mar=c(3.1,3.1,1.1,1.1))
  plot(dx(ux),prop.table(table(x$samples)),xlab="dzi.nbinom",ylab="prop.")
  abline(0,1,col="blue",lwd=2)
  par(oldpar)
}
